#include "bitmap.h"
#include <vector>
using namespace std;

int Bitmap::bFill = 64<<8;
int Bitmap::bFil2 = 128<<8;
int Bitmap::bOutl = 0;

bool Bitmap::load(string filename) {
	free();
    bmp = load_png(filename.c_str(), NULL);
    willfree = true;
	return (bmp != NULL);
}

Bitmap::Bitmap(const Bitmap& other) : bmp(NULL), willfree(false) {
    int w = other.bmp->w, h = other.bmp->h;

    bmp = create_bitmap_ex(32, w, h);
	if(bmp == NULL || other.bmp == NULL)
		return;

    for(int x = 0; x < w; ++x) {
        for(int y = 0; y < h; ++y) {
			putpixel(x, y, other.getpixel(x, y));
        }
    }

    willfree = true;
}

bool Bitmap::button(int x, int y, int x2, int y2, FONT* font, string text) {
    int fill = bFill;
    int border = bOutl;
    bool res = false;
    if(in_rect(x, y, x2, y2, mouse_x, mouse_y)) {
        fill = bFil2;
        if(mouse_b & 1) res = true;
    }
    fillrect(x, y, x2, y2, fill);
    rect(x, y, x2, y2, border);
    rect(x+1, y+1, x2-1, y2-1, border);
    textout_center(font, text.c_str(), (x+x2)/2, (y+y2)/2 - text_height(font)/2);
    return res;
}

void fade_in_anim(Bitmap& final, int speed) {
    Bitmap buffer(SCREEN_W, SCREEN_H);
    int a;
    if(speed <= 0) speed = 16;
    for(a = 0; a < 256; a += speed) {
        buffer.clear();
        set_trans_blender(0, 0, 0, a);
        draw_trans_sprite(buffer.ptr(), final.ptr(), 0, 0);
        vsync();
        buffer.draw_to_screen();
    }
    final.draw_to_screen();
}

void fade_out_anim(int speed) {
    Bitmap original(SCREEN_W, SCREEN_H);
    Bitmap buffer(SCREEN_W, SCREEN_H);
    int a;
    blit(screen, original.ptr(), 0, 0, 0, 0, SCREEN_W, SCREEN_H);
    if(speed <= 0) speed = 16;
    for(a = 255-speed; a > 0; a -= speed) {
        buffer.clear();
        set_trans_blender(0, 0, 0, a);
        draw_trans_sprite(buffer.ptr(), original.ptr(), 0, 0);
        vsync();
        buffer.draw_to_screen();
    }
    rectfill(screen, 0, 0, SCREEN_W, SCREEN_H, color(0, 0, 0));
}

