#ifndef MYALLEG_BITMAP_H_INCLUDED
#define MYALLEG_BITMAP_H_INCLUDED

#define LOAD_PNG
#include "stuff.h"

class Bitmap {
    static int bFill;
    static int bFil2;
    static int bOutl;

    BITMAP* bmp;
    bool willfree;
    void free() { if(bmp != NULL && willfree) destroy_bitmap(bmp); bmp = NULL; willfree = 0; }
public:
    Bitmap() : bmp(NULL), willfree(false) { }
    Bitmap(BITMAP* bitmap) : bmp(bitmap), willfree(false) { }
    Bitmap(int w, int h) : bmp(NULL), willfree(false) { make(w,h); }
    Bitmap(string filename) : bmp(NULL), willfree(false) { load(filename); }
    Bitmap(const Bitmap& other);
    ~Bitmap() { free(); }

    bool make(int w, int h) { free(); bmp = create_bitmap_ex(32, w, h); return willfree = (bmp!=NULL); }

    static void setbutton(int fill, int fill2, int out) { bFill = fill; bFil2 = fill2; bOutl = out; }

    // drawing primitives
    void clear(int color = 0) { clear_to_color(bmp, color); }
    void putpixel(int x, int y, int color) { ::putpixel(bmp, x, y, color); }
    int getpixel(int x, int y) const { return ::getpixel(bmp, x, y); }
    void line(int x1, int y1, int x2, int y2, int color) { ::line(bmp, x1, y1, x2, y2, color); }
    void polygon(int npoints, const int* points, int color) { ::polygon(bmp, npoints, points, color); }
    void rect(int x1, int y1, int x2, int y2, int color) { ::rect(bmp, x1, y1, x2, y2, color); }
    void fillrect(int x1, int y1, int x2, int y2, int color) { ::rectfill(bmp, x1, y1, x2, y2, color); }
    void circle(int x, int y, int r, int color) { ::circle(bmp, x, y, r, color); }
    void fillcircle(int x, int y, int r, int color) { ::circlefill(bmp, x, y, r, color); }
    void ellipse(int x, int y, int rx, int ry, int color) { ::ellipse(bmp, x, y, rx, ry, color); }
    void fillellipse(int x, int y, int rx, int ry, int color) { ::ellipsefill(bmp, x, y, rx, ry, color); }
    void floodfill(int x, int y, int color) { ::floodfill(bmp, x, y, color); }

    // draw sprites
    void draw(const Bitmap& other, int x, int y) { ::blit(other.bmp, bmp, 0, 0, x, y, other.bmp->w, other.bmp->h); }
    void draw_sprite(const Bitmap& other, int x, int y) { ::draw_sprite(bmp, other.bmp, x, y); }
    void draw_to_screen() { ::blit(bmp, screen, 0, 0, 0, 0, bmp->w, bmp->h); }

    // textout
    void textout(FONT* f, string str, int x, int y, int color = -1, int bg = -1)
        { textout_ex(bmp, f, str.c_str(), x, y, color, bg); }
    void textout_center(FONT* f, string str, int x, int y, int color = -1, int bg = -1)
        { textout_centre_ex(bmp, f, str.c_str(), x, y, color, bg); }
    void textout_right(FONT* f, string str, int x, int y, int color = -1, int bg = -1)
        { textout_right_ex(bmp, f, str.c_str(), x, y, color, bg); }
    void debug_print(string str) { textout_ex(bmp, font, str.c_str(), 0, bmp->h-text_height(font), -1, 0); }

    // other
    bool button(int x, int y, int x2, int y2, FONT* font, string text);
    bool load(string filename);
    Bitmap sub(int x, int y, int w, int h)
        { Bitmap r = create_sub_bitmap(bmp, x, y, w, h); r.willfree = true; return r; }

    BITMAP* ptr() { return bmp; }
    BITMAP* operator->() { return ptr(); }
};

inline int color(int r, int g, int b) { return makecol32(r, g, b); }
inline int color(int r, int g, int b, int a) { return makeacol32(r, g, b, a); }
inline bool in_rect(int x1, int y1, int x2, int y2, int x, int y) {
	return (x >= x1 && x <= x2 && y >= y1 && y <= y2); }
inline bool in_rect(int x1, int y1, int w, int h, int x, int y, bool) {
	return (x >= x1 && x <= x1+w && y >= y1 && y <= y1+h); }

void fade_in_anim(Bitmap& final, int speed);
void fade_out_anim(int speed);

#endif // MYALLEG_BITMAP_H_INCLUDED
