#include "Game.h"

void Game::drawIslands() {
    buffer.draw_sprite(island, 320 - island->w/2, playery - island->h/2);
    for(size_t i = 0; i < islandx.size(); ++i) {
        rotate_sprite(buffer.ptr(), island.ptr(), islandx[i]-playerx - island->w/2, islandy[i] - island->h/2, islandrot[i]);
    }
}

bool Game::touchesPlayer(int ix, int iy) {
    int dx = ix - playerx - 320;
    int dy = iy - playery;
    if(dx < 0) dx = -dx;
    if(dy < 0) dy = -dy;
    return (dx < island->w && dy < island->h);
}

void Game::handleLogic() {
    if(key[KEY_LEFT]) {
        playerx -= playerSpeed/2;
    } else if(key[KEY_RIGHT]) {
        playerx += playerSpeed;
    }
    if(key[KEY_UP]) {
        playery -= playerSpeed;
    } else if(key[KEY_DOWN]) {
        playery += playerSpeed;
    }

    if(playery < 0) playery = 0;
    if(playery > 480) playery = 480;

    for(int i = 0; i < 1; ++i) {
        if(islandx.size() < 20 || rand() < RAND_MAX/100) {
            islandx.push_back(playerx + 700 + rand() % 640);
            islandy.push_back(rand() % 480);

            islandrot.push_back(itofix(rand() % 256));
        }
    }
    if(comboTimer) comboTimer--;
    if(comboTimer == 0 && combo > 0) {
        combo = 0;
        logMessage("COMBO EXPIRED");
    }

    for(size_t i = 0; i < islandx.size(); ++i) {
        if(touchesPlayer(islandx[i], islandy[i])) {
            islandx.erase(islandx.begin() + i);
            islandy.erase(islandy.begin() + i);
            islandrot.erase(islandrot.begin() + i);
            ++combo;
            ++totalHit;
            comboTimer = 50;
            logMessage("ISLAND HIT! COMBO " + toString(combo));

            if(combo == 100) {
                logMessage("ONE HUNDRED COMBO AWARD GET");
            } else if(combo == 500) {
                logMessage("MEGA 5 HUNDRED COMBO AWARD GET");
            } else if(combo == 1000) {
                logMessage("AMAZING THOUSAND COMBO AWARD GET");
            }

            if(totalHit == 500) {
                logMessage("FIVE HUNDRED TOTAL AWARD GET");
            } else if(totalHit == 1000) {
                logMessage("ULTRA 1,000 TOTAL AWARD GET");
            } else if(totalHit == 5000) {
                logMessage("SUPER 5K TOTAL AWARD GET");
            }

            --i;
            continue;
        } else if(islandx[i] < playerx - island->w) {
            islandx.erase(islandx.begin() + i);
            islandy.erase(islandy.begin() + i);
            islandrot.erase(islandrot.begin() + i);
            if(combo > 0) {
                combo = 0;
                logMessage("C-C-C-COMBO BREAKER!");
            }
            --i;
            continue;
        }
    }
}

void Game::logMessage(string msg) {
    log.push_back(msg);
    if(log.size() > 20) {
        log.erase(log.begin());
    }
}

void Game::drawLog() {
    for(size_t i = 0; i < log.size(); ++i) {
        buffer.textout(font, log[i], buffer->w - 32 * text_length(font, "-") - 5, text_height(font) * i + 5, color(0,0,0), color(255,255,255));
    }
}

void Game::calculateFPS() {
    ++frames;
    long t = time(NULL);
    if(t > lasttime) {
        lasttime = t;
        fps = (fps + frames * 9) / 10;
        fps = floor(fps * 100) / 100;
        frames = 0;
    }
}

void Game::drawScene() {
    buffer.clear();
    buffer.draw(background, -(playerx % 640), 0);
    buffer.draw(background, -(playerx % 640) + 640, 0);
    drawIslands();
    int pow[] = {1, 2, 4, 5, 8, 10, 16, 20, 32, 40};
    static int p = 0;
    static int lhit = 0;
    if(totalHit > lhit + 10) {
        lhit = totalHit;
        p = rand() % 10;
    }
    Bitmap pixl(buffer->w / pow[p], buffer->h / pow[p]);
    pixelMove(buffer, pixl);
    pixelMove(pixl, buffer);
    drawLog();
    buffer.textout(font, toString(fps), 5, 5, color(0,0,0), color(255,255,255));
    buffer.draw_to_screen();
}

Game::Game() {
    buffer.make(SCREEN_W, SCREEN_H);
    playerx = 320;
    playery = 240;
    fps = 0.0;
    frames = 0;
    lasttime = time(NULL);
}

bool Game::loadres() {
    return titleMask.load("titleMask.png") &&
           titlePat.load("titlePat.png") &&
           background.load("background.png") &&
           island.load("island.png");
}

int Game::title() {
    int done = RET_NONE;
    int p = 0;
    while(done == RET_NONE) {
        if(closed()) done = RET_QUIT;
        buffer.clear();
        buffer.draw(titlePat, p%640, 0);
        buffer.draw(titlePat, p%640 - titlePat->w, 0);
        buffer.draw_sprite(titleMask, 0, 0);
        buffer.draw_to_screen();
        p += 5;
        if(p >= 1280) done = RET_OK;
    }
    return done;
}

volatile int Game_ticks = 0;
void Game_ticker() { ++Game_ticks; }
END_OF_FUNCTION(Game_ticker)

int Game::play() {
    LOCK_VARIABLE(Game_ticks);
    LOCK_FUNCTION(Game_ticker);
    install_int_ex(Game_ticker, BPS_TO_TIMER(60));

    combo = 0; comboTimer = 0;
    totalHit = 0;
    lasttime = time(NULL);
    int done = RET_NONE;
    logMessage("================================");
    logMessage("YOU ARE ISLAND.");
    logMessage("GO ISLAND GO!");
    while(done == RET_NONE) {
        if(closed()) done = RET_QUIT;
        if(Game_ticks > 0) {
            Game_ticks = 0;
            calculateFPS();
            handleLogic();
            drawScene();
        }
    }
    return done;
}
